(load "codegen.scm")
(load "typecheck.scm")

(define (test actual expected)
  (when (not (equal? actual expected))
    (error #f
	   (format "test failed:\nexpected: ~a\nactual:   ~a"
		   expected actual))))

(define (read-file file)
  (call-with-input-file file
    (lambda (p)
      (let loop ((next (read-char p))
                 (result ""))
        (if (eof-object? next)
            result
            (loop (read-char p) (string-append result (string next))))))))

(define (test-prog prog exit-code)
  (display prog)
  (newline)
  (compile-to-binary prog "/tmp/test-prog")
  (test (system "/tmp/test-prog") exit-code))

(define (test-prog-stdout prog output)
  (display prog)
  (newline)
  (compile-to-binary prog "/tmp/test-prog")
  (system "/tmp/test-prog > /tmp/test-output.txt")
  (let ((str (read-file "/tmp/test-output.txt")))
    (test str output)))

(test (typecheck '(lambda (x) (+ ((lambda (y) (x y 3)) 5) 2)))
      '(abs (abs int (abs int int)) int))

(test-prog '(+ 1 2) 3)
(test-prog '((lambda (x) ((lambda (y) (+ x y)) 42)) 100) 142)

(test-prog '(* 10 5) 50)

(test-prog '(let ((x (+ 1 32))
		  (y x))
	      ((lambda (z) (+ 2 z)) (* y x)))
	   67) ; exit code modulos at 256
(test-prog '(if ((lambda (x) (= x 2)) 1) 0 (- 32 1)) 31)

(test-prog-stdout '(if (= 3 2) 1 (let () (print "hello world!") 0)) "hello world!")

(test-prog '((lambda (x y) (+ x y)) 1 2) 3)
(test-prog '((lambda (x) (+ ((lambda (y) (+ y 1)) 3) x)) 2) 6)
